﻿package gamespecific
{
    import flash.display.Bitmap;
    import flash.display.Sprite;
    import flash.events.Event;
    import flash.events.HTTPStatusEvent;
    import flash.events.IOErrorEvent;
    import flash.events.MouseEvent;
    import flash.events.ProgressEvent;
    import flash.events.SecurityErrorEvent;
    import flash.net.URLLoader;
    import flash.net.URLRequest;
    import meyermike.core.GameObject;
    import meyermike.Global;
    import meyermike.widget.TextField;

    /**
     * ...
     * @author Mike Meyer
     */
    public class Planet extends GameObject
    {
        [Embed(source = './assets/loaderring.png')]
        private static const RingGraphicClass:Class;

        [Embed(source='assets/loadtext.png')]
        private static const LoadTextClass:Class;

        [Embed(source='assets/planetsurface.png')]
        private static const PlanetSurfaceClass:Class;

        [Embed(source='assets/planetshade.png')]
        private static const PlanetShadeClass:Class;

        [Embed(source='assets/planethovertext.png')]
        private static const HoverTextClass:Class;

        public static var ENTER_ATMOSPHERE:String = "enteratmosphere";

        private var mLoadingRing:GameObject;
        private var mLoadText:Bitmap;
        private var mUrlLoader:URLLoader;
        private var mStatusText:TextField;
        private var mPlanetInfo:gamespecific.PlanetInfo;

        public function get PlanetInfo():gamespecific.PlanetInfo { return mPlanetInfo; }

        private var mPlanetMask:Sprite;
        private var mPlanetSurface:Bitmap;
        private var mPlanetShade:Bitmap;
        private var mPlanetHoverText:Bitmap;
        private var mMasked:Sprite;
        private var mMouseDummy:Sprite;

        private static const CLICK_AREA_WIDTH:Number =  80;

        /**
         * initial setup and preparation for reuse
         */
        public override function Init():void
        {
            super.Init();
            CreateGraphics();
            SetupEventListeners();
        }

        /**
         * cleanup for when we are done with this instance for a while.
         * May still get reused!
         */
        public override function Shutdown():void
        {
            RemoveEventListeners();
            super.Shutdown();
        }

        /**
         *
         */
        private function CreateGraphics():void
        {
            if (null == mPlanetHoverText)
            {
                mPlanetHoverText = new HoverTextClass();
                this.addChild(mPlanetHoverText);
                mPlanetHoverText.x = -0.5 * mPlanetHoverText.width;
                mPlanetHoverText.y = -85.0 - 0.5 * mPlanetHoverText.height;
            }

            if (null == mLoadingRing)
            {
                mLoadingRing = new GameObject();
                InternalGameObjects.AddPremade(mLoadingRing);
                var gfx:Bitmap = new RingGraphicClass();
                gfx.x = -0.5 * gfx.width;
                gfx.y = -0.5 * gfx.height;
                gfx.smoothing = true;
                mLoadingRing.addChild(gfx);

                mLoadingRing.dr = 360;
            }

            if (null == mLoadText)
            {
                mLoadText = new LoadTextClass();
                this.addChild(mLoadText);
                mLoadText.x = -0.5 * mLoadText.width;
                mLoadText.y = -0.5 * mLoadText.height;
            }

            if (null == mStatusText)
            {
                mStatusText = InternalGameObjects.Add(TextField) as TextField;
                mStatusText.SetFontColor(0xFFFFFF);
                mStatusText.SetFontSize(14);
                mStatusText.CenterOn(0, 70);
            }

            if (null == mPlanetMask)
            {
                mPlanetMask = new Sprite();
                mPlanetMask.graphics.beginFill(0xFFFFFF);
                mPlanetMask.graphics.drawCircle(0, 0, 50);
                mPlanetMask.graphics.endFill();
                this.addChild(mPlanetMask);
            }

            if (null == mMasked)
            {
                mMasked = new Sprite();

                if (null == mPlanetSurface)
                {
                    mPlanetSurface = new PlanetSurfaceClass();
                    mPlanetSurface.x = -0.5 * mPlanetSurface.width;
                    mPlanetSurface.y = -0.5 * mPlanetSurface.height;
                    mMasked.addChild(mPlanetSurface);
                }

                if (null == mPlanetShade)
                {
                    mPlanetShade = new PlanetShadeClass();
                    mPlanetShade.x = -0.5 * mPlanetShade.width;
                    mPlanetShade.y = -0.5 * mPlanetShade.height;
                    mMasked.addChild(mPlanetShade);
                }

                this.addChild(mMasked);
                mMasked.mask = mPlanetMask;
            }
            if (null == mMouseDummy)
            {
                mMouseDummy = new Sprite();
                mMouseDummy.graphics.beginFill(0xFFFFFF);
                mMouseDummy.graphics.drawRect(
                    -0.5 * CLICK_AREA_WIDTH,
                    -0.5 * CLICK_AREA_WIDTH,
                    CLICK_AREA_WIDTH,
                    CLICK_AREA_WIDTH);
                mMouseDummy.graphics.endFill();
                mMouseDummy.alpha = 0;
                this.addChild(mMouseDummy);
            }

            mLoadingRing.visible = false;
            mLoadText.visible = false;
            mStatusText.visible = false;
            mPlanetHoverText.visible = false;
            mMasked.visible = true;
        }

        /**
         * set the planet up to represent a new, randomly chosen page
         */
        public function LoadRandom():void
        {
            LoadBegun();

            mUrlLoader = new URLLoader();
            mUrlLoader.addEventListener(Event.COMPLETE, RandomChoiceCompletelyDownloaded);
            mUrlLoader.addEventListener(Event.OPEN, DownloadBegun);
            mUrlLoader.addEventListener(ProgressEvent.PROGRESS, DownloadProgress);

            mUrlLoader.addEventListener(HTTPStatusEvent.HTTP_STATUS, function(e:HTTPStatusEvent):void { trace(e.toString()); } );
            mUrlLoader.addEventListener(IOErrorEvent.IO_ERROR, function(e:IOErrorEvent):void { trace(e.toString()); } );
            mUrlLoader.addEventListener(ProgressEvent.PROGRESS, function(e:ProgressEvent):void { trace(e.toString()); } );
            mUrlLoader.addEventListener(SecurityErrorEvent.SECURITY_ERROR, function(e:SecurityErrorEvent):void { trace(e.toString()); } );

            var url:String = "http://www.meyermike.com/games/minild12/relay_wikipedia_api.php?action=query&format=xml&list=random&rnlimit=10";
            var d:Date = new Date();
            url += "&cachehack="+d.valueOf();
            var request:URLRequest = new URLRequest(url);

            mStatusText.SetText("");

            try
            {
                mUrlLoader.load(request);
            }
            catch (error:Error)
            {
                trace(error);
            }
        }

        /**
         * set the planet up to represent a new page
         * @param	pageTitle - title of the wikipedia page to load
         */
        public function LoadPage(pageTitle:String):void
        {
            LoadBegun();

            mUrlLoader = new URLLoader();
            mUrlLoader.addEventListener(Event.COMPLETE, PageContentsCompletelyDownloaded);
            mUrlLoader.addEventListener(Event.OPEN, DownloadBegun);
            mUrlLoader.addEventListener(ProgressEvent.PROGRESS, DownloadProgress);

            mUrlLoader.addEventListener(HTTPStatusEvent.HTTP_STATUS, function(e:HTTPStatusEvent):void { trace(e.toString()); } );
            mUrlLoader.addEventListener(IOErrorEvent.IO_ERROR, function(e:IOErrorEvent):void { trace(e.toString()); } );
            mUrlLoader.addEventListener(ProgressEvent.PROGRESS, function(e:ProgressEvent):void { trace(e.toString()); } );
            mUrlLoader.addEventListener(SecurityErrorEvent.SECURITY_ERROR, function(e:SecurityErrorEvent):void { trace(e.toString()); } );

            var url:String = "http://www.meyermike.com/games/minild12/relay_wikipedia_api.php?action=query&titles=" + pageTitle + "&format=xml&prop=revisions|images|links&rvprop=content";
            trace("asking with url "+url);
            var request:URLRequest = new URLRequest(url);

            mStatusText.SetText("");

            try
            {
                mUrlLoader.load(request);
            }
            catch (error:Error)
            {
                trace(error);
            }
        }

        /**
         *
         */
        private function LoadBegun():void
        {
            mPlanetInfo = null;
            mLoadingRing.visible = true;
            mLoadText.visible = true;
            mStatusText.visible = true;
            mMasked.visible = false;
            mPlanetHoverText.visible = false;

            if (null != mUrlLoader)
            {
                mUrlLoader.close();
            }
        }

        /**
         * per-frame updates
         * @param	elapsedTime - how long the last frame was
         */
        public override function Update(elapsedTime:Number):void
        {
            super.Update(elapsedTime);
            mLoadingRing.Update(elapsedTime);
        }

        /**
         * event handler for when it finished successfully
         */
        private function RandomChoiceCompletelyDownloaded(e:Event):void
        {
            mUrlLoader = null;
            var xml:XML = new XML( e.target.data );

            // avoid user and talk pages
            for each (var page:XML in xml.query.random.page)
            {
                var title:String = page.@title;

                // okay a lot of characters are hosing me.  only allow ones with characters I trust
                var charactersWeDontLkeRegExp:RegExp = /^[A-Za-z0-9 _]*$/;
                if ( charactersWeDontLkeRegExp.test(title) )
                {
                    LoadPage(title.replace(/ /g, "_"));
                    return;
                }
            }

            // failed to find a good one. ask wikipedia for more
            LoadRandom();
        }

        /**
         * event handler for when it finished successfully
         */
        private function PageContentsCompletelyDownloaded(e:Event):void
        {
            mLoadingRing.visible = false;
            mLoadText.visible = false;
            mStatusText.visible = true;
            mMasked.visible = true;

            mUrlLoader = null;
            try
            {
                mPlanetInfo = new gamespecific.PlanetInfo( e.target.data );
            }
            catch (e:TypeError)
            {
                // no time to debug for reals, just find a page that works
                LoadRandom();
            }
            mStatusText.SetText("PLANET:\n  "+mPlanetInfo.Name);
        }

        /**
         * event handler for when the download is successfully started
         */
        private function DownloadBegun(e:Event):void
        {
            mStatusText.SetText("Downloading...");
        }

        //
        private function DownloadProgress(e:ProgressEvent):void
        {
            mStatusText.SetText("Downloading...\n(" + e.bytesLoaded + " / " + e.bytesTotal + ")");
        }

        /**
         * setup handlers for getting rollover effects
         */
        private function SetupEventListeners():void
        {
            addEventListener(MouseEvent.ROLL_OVER, HandleRollOver);
            addEventListener(MouseEvent.ROLL_OUT, HandleRollOut);
            addEventListener(MouseEvent.CLICK, HandleClick);
        }

        /**
         * remove rollover/rollout event handlers
         */
        private function RemoveEventListeners():void
        {
            removeEventListener(MouseEvent.ROLL_OVER, HandleRollOver);
            removeEventListener(MouseEvent.ROLL_OUT, HandleRollOut);
            removeEventListener(MouseEvent.CLICK, HandleClick);
        }

        /**
         * overloaded to make this actually register mouse lisenters to the non-spinning parts
         */
        public override function addEventListener(type:String, listener:Function, useCapture:Boolean = false, priority:int = 0, useWeakReference:Boolean = false):void
        {
            switch (type)
            {
                case MouseEvent.CLICK:
                case MouseEvent.DOUBLE_CLICK:
                case MouseEvent.MOUSE_DOWN:
                case MouseEvent.MOUSE_MOVE:
                case MouseEvent.MOUSE_OUT:
                case MouseEvent.MOUSE_OVER:
                case MouseEvent.MOUSE_UP:
                case MouseEvent.MOUSE_WHEEL:
                case MouseEvent.ROLL_OUT:
                case MouseEvent.ROLL_OVER:
                    mListeners.push( { type:type, listener:listener, useCapture:useCapture } );
                    mMouseDummy.addEventListener(type, listener, useCapture, priority, useWeakReference);
                    break;
                // other events can be done as normal
                default:
                    super.addEventListener(type, listener, useCapture, priority, useWeakReference);
                    break;
            }
        }

        /**
         * overloaded so that mouse events are registered with the non-moving part
         */
        public override function removeEventListener(type:String, listener:Function, useCapture:Boolean = false):void
        {
            switch (type)
            {
                case MouseEvent.CLICK:
                case MouseEvent.DOUBLE_CLICK:
                case MouseEvent.MOUSE_DOWN:
                case MouseEvent.MOUSE_MOVE:
                case MouseEvent.MOUSE_OUT:
                case MouseEvent.MOUSE_OVER:
                case MouseEvent.MOUSE_UP:
                case MouseEvent.MOUSE_WHEEL:
                case MouseEvent.ROLL_OUT:
                case MouseEvent.ROLL_OVER:
                    mMouseDummy.removeEventListener(type, listener, useCapture);
                    break;
                // other events can be done as normal
                default:
                    super.removeEventListener(type, listener, useCapture);
                    break;
            }
        }

        /**
         * turn on rollover effects when the mouse enters
         */
        private function HandleRollOver(evt:MouseEvent = null):void
        {
            // can't do it while downloading
            if (null == mUrlLoader)
            {
                mPlanetHoverText.visible = true;
            }
        }

        /**
         * turn off rollover effects when the mouse leaves
         */
        private function HandleRollOut(evt:MouseEvent = null):void
        {
            mPlanetHoverText.visible = false;
        }

        /**
         *
         */
        private function HandleClick(e:MouseEvent = null):void
        {
            // if we're not downloading, we can go ahead and ENTER THE ATMOSPHERE
            if (null == mUrlLoader)
            {
                dispatchEvent(new Event(ENTER_ATMOSPHERE));
            }
        }
    }

}
