﻿package gamespecific
{
    import flash.display.GradientType;
    import flash.display.Sprite;
    import flash.geom.Matrix;
    import flash.geom.Rectangle;
    import meyermike.constants.Colors;
    import meyermike.core.GameObject;
    import meyermike.core.GameObjectHolder;
    import meyermike.Global;
    import meyermike.util.MathUtil;

    /**
     * ...
     * @author Mike Meyer
     */
    public class Player extends GameObject
    {
        private var mGraphicsMade:Boolean = false;

        private static const WIDTH:Number = 64;
        private static const HEIGHT:Number = 32;

        private static const FLAME_PARTICLES_PER_FRAME:Number = 8;

        private static const LARGE_OSCILLATE_FREQUENCY:Number = 0.01;
        private static const LARGE_OSCILLATE_AMPLITUDE:Number = 0.25; // as a fraction of ScreenHeight
        private var mLargeOscillateTheta:Number = 1.5 * Math.PI;

        private static const SMALL_OSCILLATE_FREQUENCY:Number = 4.0;
        private static const SMALL_OSCILLATE_AMPLITUDE:Number = 0.005; // as a fraction of ScreenHeight
        private var mSmallOscillateTheta:Number = 0;

        private var _flameLayer:GameObjectHolder;
        public function get FlameLayer():GameObjectHolder { return _flameLayer; }
        public function set FlameLayer(value:GameObjectHolder):void { _flameLayer = value; }

        private static const FLAME_MIN_SPEED:Number = 100.0;
        private static const FLAME_MAX_SPEED:Number = 1000.0;
        private static const FLAME_ANGLE_RANDOMNESS:Number = Math.PI / 12.0;

        public override function Init():void
        {
            super.Init();

            CreateGraphics();

            PhysicsEnabled = false;
        }

        public override function Shutdown():void
        {
            super.Shutdown();
        }

        private function CreateGraphics():void
        {
            if (!mGraphicsMade)
            {
                graphics.lineStyle(1, Colors.BLACK);
                var gradientMatrix:Matrix = new Matrix();
                gradientMatrix.createGradientBox( WIDTH, HEIGHT, 0.5 * Math.PI, -0.5 * WIDTH, -0.5 * HEIGHT);
                graphics.beginGradientFill(
                    GradientType.LINEAR,
                    [Colors.WHITE, Colors.RED, Colors.BLACK],
                    [1.0, 1.0, 1.0],
                    [0, 0.6 * 255, 1.0 * 255],
                    gradientMatrix );
                graphics.moveTo( -0.5 * WIDTH, -0.5 * HEIGHT);
                graphics.lineTo( -0.25 * WIDTH, -0.25 * HEIGHT );
                graphics.lineTo(  0.125 * WIDTH, -0.25 * HEIGHT );
                graphics.lineTo(  0.5 * WIDTH,  0.25 * HEIGHT );
                graphics.lineTo(  0.25 * WIDTH,  0.375 * HEIGHT );
                graphics.lineTo(  -0.25 * WIDTH,  0.375 * HEIGHT );
                graphics.lineTo(  -0.5 * WIDTH,  0.5 * HEIGHT );
                graphics.moveTo( -0.5 * WIDTH, -0.5 * HEIGHT);
                graphics.endFill();
                mGraphicsMade = true;
            }
        }

        public override function Update(elapsedTime:Number):void
        {
            super.Update(elapsedTime);

            mLargeOscillateTheta += LARGE_OSCILLATE_FREQUENCY * 2.0 * Math.PI * elapsedTime;
            mLargeOscillateTheta = MathUtil.Wrap(mLargeOscillateTheta, 0, 2.0 * Math.PI);

            mSmallOscillateTheta += SMALL_OSCILLATE_FREQUENCY * 2.0 * Math.PI * elapsedTime;
            mSmallOscillateTheta = MathUtil.Wrap(mSmallOscillateTheta, 0, 2.0 * Math.PI);

            y =
                Global.StageHeight * (
                    0.5
                    + (LARGE_OSCILLATE_AMPLITUDE * Math.sin(mLargeOscillateTheta))
                    + (SMALL_OSCILLATE_AMPLITUDE * Math.sin(mSmallOscillateTheta)) );

            // rocket effects
            for (var i:Number = 0; i < FLAME_PARTICLES_PER_FRAME; ++i)
            {
                var flame:GameObject = FlameLayer.Add(RocketFlame);
                flame.x = x - (0.5*width);
                flame.y = y;

                var angle:Number = Math.PI + MathUtil.Lerp( -FLAME_ANGLE_RANDOMNESS, FLAME_ANGLE_RANDOMNESS, Math.random());
                var speed:Number = MathUtil.Lerp( FLAME_MIN_SPEED, FLAME_MAX_SPEED, Math.random() );
                flame.dx = speed * Math.cos(angle);
                flame.dy = speed * Math.sin(angle);
            }
        }

        /**
         * using the actual bounds looks pretty wrong
         */
        public override function GetCollisionBounds():Rectangle
        {
            return new Rectangle(
                x - (0.4 * width),
                y - (0.25 * height),
                0.8 * width,
                0.5 * height);
        }
    }

}
