﻿package gamespecific.gamestates
{
    import flash.display.LineScaleMode;
    import flash.events.Event;
    import flash.events.KeyboardEvent;
    import flash.events.MouseEvent;
    import flash.media.Sound;
    import flash.media.SoundChannel;
    import gamespecific.Bullet;
    import gamespecific.Enemy;
    import gamespecific.PlanetInfo;
    import gamespecific.Player;
    import gamespecific.RocketFlame;
    import gamespecific.step.SpawnEnemy;
    import gamespecific.step.Trace;
    import meyermike.collision.CollisionInfo;
    import meyermike.constants.Colors;
    import meyermike.core.GameInterface;
    import meyermike.core.GameObject;
    import meyermike.core.GameState;
    import meyermike.debug.FpsCounter;
    import meyermike.Global;
    import meyermike.script.Step;
    import meyermike.script.step.Condition;
    import meyermike.script.step.Loop;
    import meyermike.script.step.RunFunc;
    import meyermike.script.step.Wait;
    import meyermike.Sounds;
    import meyermike.Util;
    import meyermike.util.MathUtil;
    import meyermike.util.music.Song;
    import meyermike.widget.Fader;
    import meyermike.widget.PulsingTextButton;
    import meyermike.widget.TextField;

    /**
     * ...
     * @author Mike Meyer
     */
    public class Atmosphere extends GameState
    {
        [Embed(source='../assets/hit.mp3')]
        private static const HitSoundClass:Class;
        private static var HitSound:Sound = new HitSoundClass();
        private static var HitChannel:SoundChannel;

        [Embed(source='../assets/die.mp3')]
        private static const DieSoundClass:Class;
        private static var DieSound:Sound = new DieSoundClass();
        private static var DieChannel:SoundChannel;

        [Embed(source='../assets/shoot.mp3')]
        private static const ShootSoundClass:Class;
        private static var ShootSound:Sound = new ShootSoundClass();
        private static var ShootChannel:SoundChannel;

        private static const DISALLOWED_HEADING_REGEX:RegExp = /={2,}\s*?(References|External\s+links|Sources?)\s*?={2,}/igm;

        private static const HEADING_REGEX:RegExp = /={2,}(.*?)={2,}/gm;

        private static const STRIP_FORMATTING_REGEX:RegExp = /'{2,5}|----|^[:;\*#]+\s*|^\s+|#REDIRECT\s*|<[^>]+?>|<!--[^-->]*-->|{{([^{}]*?({{[^{}]*?}})*?[^{}]*?)*}}/gm;
        private static function StripFormatting(s:String):String { return s.replace(STRIP_FORMATTING_REGEX, ""); }

        private static const STRIP_TABLE_REGEX:RegExp = /{\|[^}]*}/gm;
        private static function StripTables(s:String):String { return s.replace(STRIP_TABLE_REGEX, ""); }

        private static const STRIP_IMAGES_REGEX:RegExp = /\[\[(File:|Image:|Media:|Special:FilePath)([^\|\]]*)[^ \]]*([^\]]*)?]]/gm;
        private static function StripImages(s:String):String { return s.replace(STRIP_IMAGES_REGEX, ""); }

        private static const STRIP_EXTERNALLINKS_REGEX:RegExp = /(?<!\[)\[([^ \[\]]+) ?([^\]]*)]/gm;
        private static function StripExternalLinks(s:String):String { return s.replace(STRIP_EXTERNALLINKS_REGEX, ""); }

        private static const STRIP_INTERNALLINKS_REGEX:RegExp = /\[\[[^|\]]*\|([^\]]+)\]\]|\[\[([^|\]]*)\]\]/gm;
        private static function StripInternalLinks(s:String):String { return s.replace(STRIP_INTERNALLINKS_REGEX, "$1$2"); }

        private static const STRIP_BLANKS_REGEX:RegExp = /[\r\n]\s*([\r\n])/gm;
        private static function StripBlanks(s:String):String { return s.replace(STRIP_BLANKS_REGEX, "$1"); }

        private static const BULLET_SPEED:Number = 1000;

        private static const CAM_SHAKE_DECAY:Number = 20.0;
        private static const HIT_TEXT_CAMSHAKE:Number = 1.75;
        private static const DESTROY_TEXT_CAMSHAKE:Number = 1.0;
        private static const LOSE_CAMSHAKE:Number = 15.0;
        private static const BULLET_IMPACT_PUSH:Number = 5.0;
        private static const FLAME_PARTICLES_TO_SPAWN_FOR_IMPACT:Number = 4;

        private static function HtmlSpecialDecode(s:String):String
        {
            return s
                .replace("&quot;", "\"")
                .replace("&amp;", "&")
                .replace("&lt;", "<")
                .replace("&gt;", ">")
                .replace("&nbsp;", " ")
                .replace("&ndash;", "-");
        }

        private static const ENTER_ATMOSPHERE_COLOR:Number = 0xFF6633;

        private var mPlanetInfo:PlanetInfo;
        private var mPlayer:Player;
        private var mFader:Fader;

        private static const WOOSH_FREQUENCY:Number = 128.0;
        private var mWooshTimer:Number = 0;
        private static const GROUND_FREQUENCY:Number = 10.0;
        private var mGroundTimer:Number = 0;

        private static const SHOOT_FREQUENCY:Number = 15.0;
        private var mShootCountdown:Number = 0;

        private var mLevelScript:Step;
        private var mPlayerDead:Boolean;

        private static var sFirstTime:Boolean = true;

        /**
         * constructor for setting the plnet info for it to use
         */
        public function Atmosphere(planetInfo:PlanetInfo)
        {
            super();

            mPlanetInfo = planetInfo;
        }

        /**
         * get the gamestate ready to be played
         */
        public override function Begin(game:GameInterface):void
        {
            super.Begin(game);

            CreatePlayer(mPlanetInfo);
            mLevelScript = CreateScript(mPlanetInfo);

            mScript = mLevelScript;
            mScript.Restart();

            Game.Music.SongList = [
                new Song("http://www.meyermike.com/games/minild12/minild12.mp3", null, null, null, 0.25) ];
            Game.Music.StartNewMusic();

            if (Game.Debug.DebuggingFeaturesAllowed)
            {
                var fpsCounter:GameObject = Game.Layers.UiLayer.Add(FpsCounter);
                fpsCounter.x = 5;
                fpsCounter.y = 5;
            }

            mFader = Game.Layers.UiLayer.Add(Fader) as Fader;
            mFader.FadeToColor(0, ENTER_ATMOSPHERE_COLOR);
            mFader.FadeBack(0.25);
            mFader.mouseEnabled = false; // let clicks, etc. go through it

            mPlayerDead = false;

            Game.stage.addEventListener(KeyboardEvent.KEY_DOWN, OnKeyDown);
        }

        // clean up after yourself
        public override function End():void
        {
            super.End();
            Game.stage.removeEventListener(KeyboardEvent.KEY_DOWN, OnKeyDown);
            Game.Layers.RemoveAll();
        }

        /**
         * per-frame stuff
         */
        public override function Update():void
        {
            super.Update();

            if (mShootCountdown > 0.0)
            {
                mShootCountdown -= Game.Time.TimeDelta;
            }

            // if the camera is shaking, let's slow it down
            if (Game.CameraShake > 0.0)
            {
                var delta:Number = CAM_SHAKE_DECAY * Game.Time.TimeDelta;
                if (Game.CameraShake < delta)
                {
                    Game.CameraShake = 0;
                }
                else
                {
                    Game.CameraShake -= delta;
                }
            }

            BackgroundWoosh();
            Ground();

            if (!mPlayerDead)
            {
                ProjectilesHittingEnemies();
                PlayerEnemyCollision();
            }
        }

        /**
         *
         */
        private function BackgroundWoosh():void
        {
            mWooshTimer += WOOSH_FREQUENCY * Game.Time.TimeDelta;

            while (mWooshTimer > 0.0)
            {
                Game.Layers.BackgroundLayer.Add(gamespecific.BackgroundWoosh);
                mWooshTimer -= 1.0;
            }
        }

        /**
         *
         */
        private function Ground():void
        {
            mGroundTimer += GROUND_FREQUENCY * Game.Time.TimeDelta;

            while (mGroundTimer > 0.0)
            {
                Game.Layers.BackgroundLayer.Add(gamespecific.GroundWoosh);
                mGroundTimer -= 1.0;
            }
        }

        /**
         * make the player
         */
        private function CreatePlayer(planetInfo:PlanetInfo):void
        {
            mPlayer = Game.Layers.PlayerLayer.Add(Player) as Player;
            mPlayer.x = 0.2 * Global.StageWidth;
            mPlayer.y = 0.1 * Global.StageHeight;
            mPlayer.FlameLayer = Game.Layers.NpcLayer;
        }

        /**
         * Set up the script that drives the level
         */
        private function CreateScript(planetInfo:PlanetInfo):Step
        {
            var script:Array = [];

            var cleanedContents:String =
                HtmlSpecialDecode(
                    StripBlanks(
                        StripInternalLinks(
                            StripExternalLinks(
                                StripImages(
                                        StripTables(
                                            StripFormatting(
                                                planetInfo.PageContents ) ) ) ) ) ) );

            if (sFirstTime)
            {
                sFirstTime = false;
                cleanedContents = "Type to shoot at the nearest words containing the character you type."
                                + "Bigger words take more hits to destroy."
                                + cleanedContents;
            }
            for each ( var line:String in cleanedContents.split('\n') )
            {
                // some sections are stupid.  ignore 'em
                DISALLOWED_HEADING_REGEX.lastIndex = 0;
                if ( DISALLOWED_HEADING_REGEX.test(line) )
                {
                    break;
                }

                for each (var sentence:String in line.split("."))
                {
                    if (sentence.length > 0)
                    {
                        var words:Array
                        HEADING_REGEX.lastIndex = 0;
                        if (HEADING_REGEX.test(sentence))
                        {
                            words = [sentence.replace(HEADING_REGEX, "$1")];
                        }
                        else
                        {
                            words = sentence.split(/[\s\r\n]+/gm);
                        }

                        for ( var i:Number = 0; i < words.length; ++i )
                        {
                            var word:String = words[i];

                            if (word.length > 0) // dunno how I'm getting empty strings in here, but geeze
                            {
                                if ( i == (words.length - 1) )
                                {
                                    word += ".";
                                }
                                script.push( new SpawnEnemy(Game.Layers.EnemyLayer, word) );
                            }
                        }
                        script.push( new Wait( 6 ) );
                    }
                }

            }
            script.push( new Condition(AllEnemiesGone) );
            script.push( new RunFunc(ShowWinMenu) );

            return new Loop(1, script);
        }

        /**
         * used in a condition step to make the game not end until the enemies are all dealt with
         */
        private function AllEnemiesGone():Boolean
        {
            return (0==Game.Layers.EnemyLayer.Objects.length);
        }

        /**
         * when he wins, have a restart / leaderboards / back to orbit button
         */
        private function ShowWinMenu():void
        {
            if (!mPlayerDead)
            {
                mFader.FadeToColor(0.5, Colors.BLACK);
                var tf:TextField = Game.Layers.UiLayer.Add(TextField) as TextField;
                tf.SetText(mPlanetInfo.Name + " defeated!");
                tf.CenterOn( 0.5 * Global.StageWidth, 0.3 * Global.StageHeight );

                var btn:PulsingTextButton = Game.Layers.UiLayer.Add(PulsingTextButton) as PulsingTextButton;
                btn.SetText("Next Planet");
                btn.SetFontColor(Colors.WHITE_ENOUGH);
                btn.CenterOn( 0.5 * Global.StageWidth, 0.6 * Global.StageHeight );
                btn.addEventListener(MouseEvent.CLICK, ReturnToSpace);
            }
        }
        private function ReturnToSpace(e:Event = null):void
        {
            Game.ChangeGameState(new PlanetSelect());
        }

        /**
         * when he dies, have a restart / back to orbit button
         */
        private function ShowLoseMenu():void
        {
            mFader.FadeToColor(0.5, Colors.BLACK);
            var tf:TextField = Game.Layers.UiLayer.Add(TextField) as TextField;
            tf.SetText("You have failed.");
            tf.CenterOn( 0.5 * Global.StageWidth, 0.3 * Global.StageHeight );

            var btn:PulsingTextButton = Game.Layers.UiLayer.Add(PulsingTextButton) as PulsingTextButton;
            btn.SetText("Try This Planet Again");
            btn.SetFontColor(Colors.WHITE_ENOUGH);
            btn.CenterOn( 0.5 * Global.StageWidth, 0.6 * Global.StageHeight );
            btn.addEventListener(MouseEvent.CLICK, function(e:Event):void { Restart(); } );

            btn = Game.Layers.UiLayer.Add(PulsingTextButton) as PulsingTextButton;
            btn.SetText("Try a Different Planet");
            btn.SetFontColor(Colors.WHITE_ENOUGH);
            btn.CenterOn( 0.5 * Global.StageWidth, 0.8 * Global.StageHeight );
            btn.addEventListener(MouseEvent.CLICK, ReturnToSpace);
        }

        private function OnKeyDown(e:KeyboardEvent):void
        {
            if (!mPlayerDead)
            {
                if (mShootCountdown <= 0.0)
                {
                    for each (var enemy:Enemy in Game.Layers.EnemyLayer.Objects)
                    {
                        if (!enemy.Dying)
                        {
                            if ( -1 != enemy.Text.toLowerCase().indexOf(String.fromCharCode(e.charCode).toLowerCase()))
                            {
                                ShootAt(enemy);
                                break; // stop on first target
                            }
                        }
                    }

                    mShootCountdown = 1.0 / SHOOT_FREQUENCY;
                }
            }
        }

        private function ShootAt(enemy:Enemy):void
        {
            var b:GameObject= Game.Layers.BulletLayer.Add(Bullet);
            b.x = mPlayer.x + (0.5 * mPlayer.width);
            b.y = mPlayer.y;

            var direction:Number = Math.atan2(enemy.y - b.y, enemy.x - b.x);
            b.rotation = direction * 180.0 / Math.PI;

            b.dx = BULLET_SPEED * Math.cos(direction);
            b.dy = BULLET_SPEED * Math.sin(direction);

            ShootChannel = Sounds.PlaySound(ShootSound, ShootChannel, 0.5);
        }

        private function ProjectilesHittingEnemies():void
        {
            Game.Layers.ForEachCartesianProductStyle(Game.Layers.BulletLayer, Game.Layers.EnemyLayer, BulletEnemyCheck);
        }
        private function BulletEnemyCheck(b:Bullet, e:Enemy):void
        {
            if (!e.Dying)
            {
                var ci:CollisionInfo = Util.CheckAabbCollision(b, e, Game.Time.TimeDelta);
                if (null != ci)
                {
                    // hack to kill it NO TIME LEFT
                    b.x = 10000;
                    e.hp -= 1;

                    var direction:Number = Math.atan2(b.y - e.y, b.x - e.x);

                    // shake a bit
                    if (e.hp > 0)
                    {
                        Game.CameraShake += HIT_TEXT_CAMSHAKE;

                        // push the enemy back
                        e.x += BULLET_IMPACT_PUSH * Math.cos(direction);
                        e.y += BULLET_IMPACT_PUSH * Math.sin(direction);
                        HitEffects(ci.location.x, ci.location.y, direction);
                        HitChannel = Sounds.PlaySound(HitSound, HitChannel, 0.25);
                    }
                    else
                    {
                        Game.CameraShake += DESTROY_TEXT_CAMSHAKE;
                        HitEffects(ci.location.x, ci.location.y, direction);
                        HitEffects(e.x, e.y, -0.5 * Math.PI);
                        HitEffects(e.x, e.y, -0.5 * Math.PI);
                        HitEffects(e.x, e.y, -0.5 * Math.PI);
                        DieChannel = Sounds.PlaySound(DieSound, DieChannel, 0.5);
                    }

                }
            }
        }

        private function PlayerEnemyCollision():void
        {
            Game.Layers.ForEachCartesianProductStyle(Game.Layers.PlayerLayer, Game.Layers.EnemyLayer, PlayerEnemyCheck);
        }
        private function PlayerEnemyCheck(p:Player, e:Enemy):void
        {
            if (!e.Dying)
            {
                var ci:CollisionInfo = Util.CheckAabbCollision(p, e, Game.Time.TimeDelta);
                if (null != ci)
                {
                    e.hp = 0;
                    Game.CameraShake += LOSE_CAMSHAKE;
                    HitEffects(ci.location.x, ci.location.y, 0);
                    HitEffects(p.x, p.y, -0.5 * Math.PI);
                    HitEffects(p.x, p.y, -0.5 * Math.PI);
                    HitEffects(p.x, p.y, -0.5 * Math.PI);
                    DieChannel = Sounds.PlaySound(DieSound, DieChannel, 0.5);
                    mPlayerDead = true;
                    mLevelScript = null;
                    ShowLoseMenu();
                }
            }
        }


        private function HitEffects(x:Number, y:Number, direction:Number):void
        {
            for (var i:Number = 0; i < FLAME_PARTICLES_TO_SPAWN_FOR_IMPACT; ++i)
            {
                var flame:GameObject = Game.Layers.BloodLayer.Add(RocketFlame);
                flame.x = x;
                flame.y = y;

                var angle:Number = direction + MathUtil.Lerp( -0.1 * Math.PI, 0.1 * Math.PI, Math.random());
                var speed:Number = Math.random() * 600;
                flame.dx = speed * Math.cos(angle);
                flame.dy = speed * Math.sin(angle);
            }
        }
    }

}
