﻿package gamespecific.gamestates
{
    import flash.display.Bitmap;
    import flash.events.Event;
    import flash.events.MouseEvent;
    import flash.media.Sound;
    import flash.media.SoundChannel;
    import gamespecific.Planet;
    import gamespecific.widget.WarpButton;
    import meyermike.core.GameInterface;
    import meyermike.core.GameObject;
    import meyermike.core.GameState;
    import meyermike.debug.FpsCounter;
    import meyermike.Global;
    import meyermike.script.step.Condition;
    import meyermike.script.step.Loop;
    import meyermike.script.step.RunFunc;
    import meyermike.Sounds;
    import meyermike.util.MathUtil;
    import meyermike.util.music.Song;
    import meyermike.widget.Fader;
    import meyermike.widget.PulsingTextButton;

    /**
     * ...
     * @author Mike Meyer
     */
    public class PlanetSelect extends GameState
    {
        [Embed(source='../assets/warp.mp3')]
        private static const WarpSoundClass:Class;
        private static var WarpSound:Sound = new WarpSoundClass();

        [Embed(source='../assets/enteratmosphere.mp3')]
        private static const EnterAtmosphereSoundClass:Class;
        private static var EnterAtmosphereSound:Sound = new EnterAtmosphereSoundClass();

        private static var SoundEffectsChannel:SoundChannel;

        private static const WARP_COLOR:Number = 0xFFFFFF;
        private static const ENTER_ATMOSPHERE_COLOR:Number = 0xFF6633;

        private var mPlanet:Planet;
        private var mFader:Fader;
        private var mStarField:GameObject;
        private var mWarpButton:WarpButton;

        private static const STARFIELD_PIECES:Number = 600;

        // maximum offset for the planet and warp
        private static const MAX_OFFSET:Number = 50;

        // scale for th camera shake
        private static const CAMERA_SHAKE:Number = 10;

        // set up the gamestate
        public override function Begin(game:GameInterface):void
        {
            super.Begin(game);

            mStarField = Game.Layers.UiLayer.Add(GameObject);
            RandomizeStarField();

            mWarpButton = Game.Layers.UiLayer.Add(WarpButton) as WarpButton;
            mWarpButton.addEventListener(MouseEvent.CLICK, Warp);

            mPlanet = Game.Layers.UiLayer.Add(Planet) as Planet;
            mPlanet.LoadRandom();
            mPlanet.addEventListener(Planet.ENTER_ATMOSPHERE, EnterAtmosphere);

            mFader = Game.Layers.UiLayer.Add(Fader) as Fader;
            mFader.FadeToColor(0, 0x000000);
            mFader.FadeBack(2.0);
            mFader.mouseEnabled = false; // let clicks, etc. go through it

            if (Game.Debug.DebuggingFeaturesAllowed)
            {
                var fpsCounter:GameObject = this.Game.Layers.UiLayer.Add(FpsCounter);
                fpsCounter.x = 5;
                fpsCounter.y = 5;
            }

            PositionThings();

            Game.Music.SongList = [
                new Song("http://www.meyermike.com/games/minild12/minild12menu.mp3", null, null, null, 0.25) ];
            Game.Music.StartNewMusic();
        }

        /**
         * randomize the positions of the planet and warp
         */
        private function PositionThings():void
        {
            var offsetDirection:Number = MathUtil.Lerp(0, 2.0 * Math.PI, Math.random() );
            var offsetAmount:Number = MathUtil.Lerp(0, MAX_OFFSET, Math.random() );

            var warpBaseX:Number = 0.30 * Global.StageWidth;
            var warpBaseY:Number = 0.30 * Global.StageHeight;
            mWarpButton.x = warpBaseX  + (offsetAmount * Math.cos(offsetDirection));
            mWarpButton.y = warpBaseY + (offsetAmount * Math.sin(offsetDirection));

            offsetDirection = MathUtil.Lerp(0, 2.0 * Math.PI, Math.random() );
            offsetAmount = MathUtil.Lerp(0, MAX_OFFSET, Math.random() );

            var planetBaseX:Number = 0.55 * Global.StageWidth;
            var planetBaseY:Number = 0.55 * Global.StageHeight;
            mPlanet.x = planetBaseX + (offsetAmount * Math.cos(offsetDirection));
            mPlanet.y = planetBaseY + (offsetAmount * Math.sin(offsetDirection));
        }

        // clean up after yourself
        public override function End():void
        {
            super.End();
            Game.Layers.RemoveAll();
        }


        /**
         * go to a new random planet
         */
        private function Warp(e:Event = null):void
        {
            BeginWarpEffects();
        }

        /**
         *
         */
        private function BeginWarpEffects():void
        {
            mFader.FadeToColor(0.125, WARP_COLOR, DoWarp);
            SoundEffectsChannel = Sounds.PlaySound(WarpSound, SoundEffectsChannel, 0.25);
        }

        /**
         * fading to a color ended, fade back
         */
        private function DoWarp():void
        {
            mPlanet.LoadRandom();
            RandomizeStarField();
            PositionThings();
            mFader.FadeBack(0.5);
        }

        /**
         * begin transition to planet
         */
        private function EnterAtmosphere(e:Event):void
        {
            mPlanet.removeEventListener(Planet.ENTER_ATMOSPHERE, EnterAtmosphere);
            mWarpButton.removeEventListener(MouseEvent.CLICK, Warp);

            mFader.FadeToColor(1.0, ENTER_ATMOSPHERE_COLOR, DoStateSwitch);

            Game.Music.StopMusic();

            SoundEffectsChannel = Sounds.PlaySound(EnterAtmosphereSound, SoundEffectsChannel, 0.5);
            Game.CameraShake = CAMERA_SHAKE;
        }

        /**
         * ATMOSPHERE ENTERED
         */
        private function DoStateSwitch():void
        {
            Game.ChangeGameState(new Atmosphere(mPlanet.PlanetInfo));
            Game.CameraShake = 0;
        }

        /**
         *
         */
        private function RandomizeStarField():void
        {
            mStarField.graphics.clear();
            for (var i:Number = 0; i < STARFIELD_PIECES; ++i )
            {
                var x:Number = Global.StageWidth * Math.random();
                var y:Number = Global.StageHeight * Math.random();
                mStarField.graphics.beginFill(0xFFFFFF, MathUtil.Lerp(0.25, 0.5, Math.random()));
                mStarField.graphics.drawRect(x - 1, y - 1, 2, 2);
                mStarField.graphics.endFill();
            }
        }
    }

}
