﻿package gamespecific.widget
{
    import flash.display.Bitmap;
    import flash.display.Sprite;
    import flash.events.Event;
    import flash.events.MouseEvent;
    import meyermike.core.GameObject;
    import meyermike.util.MathUtil;

    /**
     * warp through space and time! Rather, a fancy button for picking a new random page.
     * @author Mike Meyer
     */
    public class WarpButton extends GameObject
    {
        [Embed(source='../assets/WarpLayer1.png')]
        private static var Layer1Class:Class;
        [Embed(source='../assets/WarpLayer2.png')]
        private static var Layer2Class:Class;
        [Embed(source='../assets/WarpText.png')]
        private static var WarpTextClass:Class;

        private var mLayer1:Sprite;
        private var mLayer2:Sprite;
        private var mTextLayer:Sprite;
        private var mMouseDummy:Sprite;

        private static const LAYER1_ROTATE_SPEED:Number = -90; // degrees / second
        private static const LAYER2_ROTATE_SPEED:Number =  90; // degrees / second

        private static const CLICK_AREA_WIDTH:Number =  60;

        private static const SIZE_PULSE_RATE1:Number = 0.309; // cycles / second
        private static const SIZE_PULSE_RATE2:Number = 0.382; // cycles / second
        private static const SIZE_MIN:Number = 0.75;
        private static const SIZE_MAX:Number = 1.0;
        private var mSizePulseTheta1:Number;
        private var mSizePulseTheta2:Number;

        private static const ALPHA_PULSE_RATE1:Number = 0.146; // cycles / second
        private static const ALPHA_PULSE_RATE2:Number = 0.236; // cycles / second
        private static const ALPHA_MIN:Number = 0.5;
        private static const ALPHA_MAX:Number = 1.0;
        private var mAlphaPulseTheta1:Number;
        private var mAlphaPulseTheta2:Number;

        /**
         * setup after first created or reused
         */
        public override function Init():void
        {
            super.Init();

            CreateGraphics();
            SetupEventListeners();

            mSizePulseTheta1 = 0;
            mSizePulseTheta2 = 0;
            mAlphaPulseTheta1 = 0;
            mAlphaPulseTheta2 = 0;
        }

        /**
         * cleanup for when we are done with this instance for a while.
         * May still get reused!
         */
        public override function Shutdown():void
        {
            RemoveEventListeners();
            super.Shutdown();
        }

        /**
         * make the images if needed
         */
        private function CreateGraphics():void
        {
            if (null == mLayer1)
            {
                mLayer1 = CreateCenteredBitmap(Layer1Class);
            }
            if (null == mLayer2)
            {
                mLayer2 = CreateCenteredBitmap(Layer2Class);
            }
            if (null == mTextLayer)
            {
                mTextLayer = CreateCenteredBitmap(WarpTextClass);
                mTextLayer.y -= 0.5 * mTextLayer.height;
            }
            if (null == mMouseDummy)
            {
                mMouseDummy = new Sprite();
                mMouseDummy.graphics.beginFill(0xFFFFFF);
                mMouseDummy.graphics.drawRect(
                    -0.5 * CLICK_AREA_WIDTH,
                    -0.5 * CLICK_AREA_WIDTH,
                    CLICK_AREA_WIDTH,
                    CLICK_AREA_WIDTH);
                mMouseDummy.graphics.endFill();
                mMouseDummy.alpha = 0;
                this.addChild(mMouseDummy);
            }

            mTextLayer.visible = false;
        }

        /**
         * helper for CreateGraphics
         * Creates a bitmap for the given class, puts it in a holder movieclip
         * and returns that
         */
        private function CreateCenteredBitmap(bitmapClass:Class):Sprite
        {
            var s:Sprite = new Sprite();
            this.addChild(s);

            var gfx:Bitmap = new bitmapClass();
            gfx.x = -0.5 * gfx.width;
            gfx.y = -0.5 * gfx.height;
            s.addChild(gfx);

            return s;
        }

        /**
         * setup handlers for getting rollover effects
         */
        private function SetupEventListeners():void
        {
            addEventListener(MouseEvent.ROLL_OVER, HandleRollOver);
            addEventListener(MouseEvent.ROLL_OUT, HandleRollOut);
        }

        /**
         * remove rollover/rollout event handlers
         */
        private function RemoveEventListeners():void
        {
            removeEventListener(MouseEvent.ROLL_OVER, HandleRollOver);
            removeEventListener(MouseEvent.ROLL_OUT, HandleRollOut);
        }


        /**
         * overloaded to make this actually register mouse lisenters to the non-spinning parts
         */
        public override function addEventListener(type:String, listener:Function, useCapture:Boolean = false, priority:int = 0, useWeakReference:Boolean = false):void
        {
            switch (type)
            {
                case MouseEvent.CLICK:
                case MouseEvent.DOUBLE_CLICK:
                case MouseEvent.MOUSE_DOWN:
                case MouseEvent.MOUSE_MOVE:
                case MouseEvent.MOUSE_OUT:
                case MouseEvent.MOUSE_OVER:
                case MouseEvent.MOUSE_UP:
                case MouseEvent.MOUSE_WHEEL:
                case MouseEvent.ROLL_OUT:
                case MouseEvent.ROLL_OVER:
                    mListeners.push( { type:type, listener:listener, useCapture:useCapture } );
                    mMouseDummy.addEventListener(type, listener, useCapture, priority, useWeakReference);
                    break;
                // other events can be done as normal
                default:
                    super.addEventListener(type, listener, useCapture, priority, useWeakReference);
                    break;
            }
        }

        /**
         * overloaded so that mouse events are registered with the non-moving part
         */
        public override function removeEventListener(type:String, listener:Function, useCapture:Boolean = false):void
        {
            switch (type)
            {
                case MouseEvent.CLICK:
                case MouseEvent.DOUBLE_CLICK:
                case MouseEvent.MOUSE_DOWN:
                case MouseEvent.MOUSE_MOVE:
                case MouseEvent.MOUSE_OUT:
                case MouseEvent.MOUSE_OVER:
                case MouseEvent.MOUSE_UP:
                case MouseEvent.MOUSE_WHEEL:
                case MouseEvent.ROLL_OUT:
                case MouseEvent.ROLL_OVER:
                    mMouseDummy.removeEventListener(type, listener, useCapture);
                    break;
                // other events can be done as normal
                default:
                    super.removeEventListener(type, listener, useCapture);
                    break;
            }
        }

        /**
         * turn on rollover effects when the mouse enters
         */
        private function HandleRollOver(evt:MouseEvent = null):void
        {
            mTextLayer.visible = true;
        }

        /**
         * turn off rollover effects when the mouse leaves
         */
        private function HandleRollOut(evt:MouseEvent = null):void
        {
            mTextLayer.visible = false;
        }

        /**
         * per-frame updates
         */
        public override function Update(elapsedTime:Number):void
        {
            super.Update(elapsedTime);

            mLayer1.rotation += LAYER1_ROTATE_SPEED * elapsedTime;
            mLayer2.rotation += LAYER2_ROTATE_SPEED * elapsedTime;

            var lerpVal:Number;

            mSizePulseTheta1 += SIZE_PULSE_RATE1 * 2.0 * Math.PI * elapsedTime;
            lerpVal = 0.5 + 0.5 * Math.sin(mSizePulseTheta1);
            mLayer1.scaleX = MathUtil.Lerp(SIZE_MIN, SIZE_MAX, lerpVal);
            mLayer1.scaleY = mLayer1.scaleX;

            mSizePulseTheta2 += SIZE_PULSE_RATE2 * 2.0 * Math.PI * elapsedTime;
            lerpVal = 0.5 + 0.5 * Math.sin(mSizePulseTheta2);
            mLayer2.scaleX = MathUtil.Lerp(SIZE_MIN, SIZE_MAX, lerpVal);
            mLayer2.scaleY = mLayer2.scaleX;

            mAlphaPulseTheta1 += ALPHA_PULSE_RATE1 * 2.0 * Math.PI * elapsedTime;
            lerpVal = 0.5 + 0.5 * Math.sin(mAlphaPulseTheta1);
            mLayer1.alpha = MathUtil.Lerp(ALPHA_MIN, ALPHA_MAX, lerpVal);

            mAlphaPulseTheta2 += ALPHA_PULSE_RATE2 * 2.0 * Math.PI * elapsedTime;
            lerpVal = 0.5 + 0.5 * Math.sin(mAlphaPulseTheta2);
            mLayer2.alpha = MathUtil.Lerp(ALPHA_MIN, ALPHA_MAX, lerpVal);
        }
    }

}
