﻿package meyermike
{
    import flash.events.Event;
    import flash.geom.Point;
    import flash.geom.Rectangle;
    import meyermike.collision.CollisionInfo;
    import meyermike.core.GameObject;

    /**
     * ...
     * @author Mike Meyer
     */
    public class PlatformerCharacter extends GameObject
    {
        public static var EVENT_DEATH:String = "death";

        private var mDying:Boolean;
        public function get Dying():Boolean { return mDying; }
        public function set Dying(value:Boolean):void { mDying = value; }

        private var mIsOnGround:Boolean;
        public function get IsOnGround():Boolean { return mIsOnGround; }

        // direction options
        protected static const LEFT:Number = -1;
        protected static const STOP:Number = 0;
        protected static const RIGHT:Number = 1;

        /// values for of platformer character acceleration
        /// subclasses will want to override these
        protected var mWalkAccel:Number = 600;
        protected var mAirAccel:Number = 600;
        protected var mStopAccel:Number = 900;
        protected var mMaxWalkSpeed:Number = 300;

        /**
         * set initial values after spawning
         */
        protected function Restart():void
        {
            mIsOnGround = false;
            PhysicsEnabled = true;
            mDying = false;
            this.dr = 0;
            this.rotation = 0;
        }

        /**
         * Start doing the dying animation
         */
        public function StartDeath():void
        {
            PhysicsEnabled = false;
            mDying = true;
            dispatchEvent(new Event(EVENT_DEATH));
        }

        /**
         * Setup that needs to happen before we do the collision checks
         * During collision checks, CollisionOccurred can get called many times.
         * Afterward, we go to PostCollisionCheck to make the actual movement happen.
         */
        public function PreCollisionCheck(timeDelta:Number):void
        {
            mIsOnGround = false;
        }

        /**
         * override to deal with collisions
         */
        public function CollisionOccurred(ci:CollisionInfo, object:Object):void
        {
            var collisionBox:Rectangle = this.GetCollisionBounds();

            this.x = ci.location.x;
            if ( 0.0 < ci.normal.x )
            {
                this.x += (0.5 * collisionBox.width);
            }
            else if ( 0.0 > ci.normal.x )
            {
                this.x -= (0.5 * collisionBox.width);
            }

            this.y = ci.location.y;
            if ( 0.0 < ci.normal.y )
            {
                this.y += (0.5 * collisionBox.height);
            }
            else if ( 0.0 > ci.normal.y )
            {
                this.y -= (0.5 * collisionBox.height);
            }

            if (ci.normal.x != 0.0)
            {
                this.dx = 0.0;
            }
            if (ci.normal.y != 0.0)
            {
                this.dy = 0.0;
            }

            mIsOnGround = mIsOnGround || (ci.normal.y < -Math.SQRT1_2);
        }

        /**
         *
         */
        public function PostCollisionCheck(timeDelta:Number):void
        {
            super.Integrate(timeDelta);
        }

        /**
         * platformer characters need to do this in postcollision instead
         * reason:  input happens between collisions and integrate.
         */
        public override function Integrate(elapsedTime:Number):void
        {
        }

        /**
         * Handle the math to do the right kinds of acceleration
         * for platform character movement
         */
        protected function WalkAccel(desiredDirection:Number, elapsedTime:Number):void
        {
            var currentDirection:Number = 0;
            if (this.dx > 0)
            {
                currentDirection = RIGHT;
            }
            else if (this.dx < 0)
            {
                currentDirection = LEFT;
            }

            // controls a bit differently on the ground
            if (IsOnGround)
            {
                // changing direction or stopping
                if ( (currentDirection != 0) && (desiredDirection != currentDirection) )
                {
                    var ddx:Number = mStopAccel * elapsedTime;

                    if ( ddx > Math.abs(this.dx) )
                    {
                        this.dx = 0;
                    }
                    else
                    {
                        if (this.dx > 0 )
                        {
                            this.dx -= ddx;
                        }
                        else
                        {
                            this.dx += ddx;
                        }
                    }
                }
                else // accelerating
                {
                    this.dx += desiredDirection * mWalkAccel * elapsedTime;
                }
            }
            else
            {
                this.dx += desiredDirection * mAirAccel * elapsedTime;
            }

            // max speed
            if (this.dx > mMaxWalkSpeed)
            {
                this.dx = mMaxWalkSpeed;
            }
            if (this.dx < -mMaxWalkSpeed)
            {
                this.dx = -mMaxWalkSpeed;
            }

            // flip graphics
            if (desiredDirection > 0)
            {
                this.scaleX = 1.0;
            }
            if (desiredDirection < 0)
            {
                this.scaleX = -1.0;
            }

        }
    }
}
