﻿package meyermike.core
{
    import flash.display.DisplayObjectContainer;
    import flash.display.Stage;
    import flash.geom.Point;
    import meyermike.core.GameState;
    import meyermike.debug.DebugWidget;
    import meyermike.util.CheatUtil;
    import meyermike.util.music.MusicUtil;
    import meyermike.util.SceneLayers;
    import meyermike.util.TimeUtil;

    /**
     * common interface to game implementations
     * @author Mike Meyer
     */
    public interface GameInterface
    {
        /**
         * The Stage that the game is attached to.
         */
        function get stage () : Stage

        /**
         * target to scroll towards
         */
        //function get CameraFollowTarget():GameObject

        /**
         * give it a target to scroll towards
         */
        //function set CameraFollowTarget(go:GameObject):void

        /**see how much the game is zoomed in
         */
        function get Zoom():Number

        /**
         * set how much the game is zoomed in
         */
        function set Zoom(zoom:Number):void

        /**
         * Allow access to the "camera" position, so that objects can follow the camera.
         * NOTE this actually deals with the position of the upper-left corner of the screen
         */
        function get CameraPosition():Point
        function set CameraPosition(pos:Point):void

        /**
         * Get and set the camera shake amount.
         * You want this to be zero most of the time!
         */
        function get CameraShake():Number
        function set CameraShake(value:Number):void

        /**
         * Get a reference to the music utility
         */
        function get Music():MusicUtil

        /**
         * Get a reference to the cheat utility
         */
        function get Cheats():CheatUtil

        /**
         * Get a reference to the SceneLayers utility
         */
        function get Layers():SceneLayers

        /**
         * Get a reference to the time utility
         */
        function get Time():TimeUtil

        /**
         * access to debugging features
         */
        function get Debug():DebugWidget

        /**
         * re-init.  Will get called automagically the first time.
         */
        function Init():void

        /**
         * freeze time and show the pause screen
         */
        function Pause():void

        /**
         * resume after pausing
         */
        function Unpause():void

        /**
         * set a state to switch to once the current beat/update is finished
         */
        function ChangeGameState(nextState:GameState):void

        /**
         * clean up anything this coul dpossibly leave dangling
         */
        function Shutdown():void
    }
}
