﻿package meyermike.core
{
    import flash.display.MovieClip;
    import flash.display.Sprite;
    import flash.events.Event;
    import flash.geom.Rectangle;

    /**
     * ...
     * @author Mike Meyer
     */
    public class GameObject extends MovieClip
    {
        public var dx:Number = 0.0;
        public var dy:Number = 0.0;
        public var dr:Number = 0.0;
        protected var mCurBeat:Number;

        /// workaround for Flash rounding to twips
        private var _x:Number = 0.0;
        public override function get x () : Number { return _x; }
        public override function set x (value:Number) : void { _x = value; super.x = value; }

        /// workaround for Flash rounding to twips
        private var _y:Number = 0.0;
        public override function get y () : Number { return _y; }
        public override function set y (value:Number) : void { _y = value; super.y = value; }

        /// so that physics utilities can tell what they need to ignore
        private var mIsPhysicsEnabled:Boolean;
        public function get PhysicsEnabled():Boolean { return mIsPhysicsEnabled; }
        public function set PhysicsEnabled(value:Boolean):void { mIsPhysicsEnabled = value; }

        /**
         * GameObjectHolder for managing internal gameobjects
         */
        private var _holder:GameObjectHolder;
        protected function get InternalGameObjects():GameObjectHolder { return _holder; }

        protected var mListeners:Array;

        /**
         * initial setup and preparation for reuse
         */
        public function Init():void
        {
            mCurBeat = 0;
            this.x = 0.0;
            this.y = 0.0;
            this.dx = 0.0;
            this.dy = 0.0;
            mIsPhysicsEnabled = true;
            mListeners = new Array();
            _holder = new GameObjectHolder(this);
        }

        public function Integrate(elapsedTime:Number):void
        {
            this.x += dx * elapsedTime;
            this.y += dy * elapsedTime;
            this.rotation += dr * elapsedTime;

            InternalGameObjects.Integrate(elapsedTime);
        }

        /**
         * per-frame updates
         * @param	elapsedTime - how long the last frame was
         */
        public function Update(elapsedTime:Number):void
        {
            InternalGameObjects.Update(elapsedTime);
        }

        public function Beat(beat:Number):void
        {
            ++mCurBeat;
            InternalGameObjects.Beat(beat);
        }

        public function OffBeat(beat:Number):void
        {
            InternalGameObjects.OffBeat(beat);
        }

        public function QuarterBeat(quarterBeat:Number):void
        {
            InternalGameObjects.QuarterBeat(quarterBeat);
        }

        /**
         * called after all the final positions have been figured
         * just before drawing
         */
        public function PreDraw():void
        {
            InternalGameObjects.PreDraw();
        }

        /**
         * Get the collision bounds for collision testing
         */
        public function GetCollisionBounds():Rectangle
        {
            return this.getBounds(this.root);
        }

        /**
         * by default a GO never dies
         */
        public function IsAlive():Boolean
        {
            return true;
        }

        /**
         * cleanup for when we are done with this instance for a while.
         * May still get reused!
         */
        public function Shutdown():void
        {
            RemoveAllListeners();
        }

        /**
         * for spherical collision tests
         */
        public function GetCollisionRadius():Number
        {
            return 0.5 * this.width;
        }

        /**
         * overloaded so that we can automagically clean up listeners in shutdown
         */
        public override function addEventListener(type:String, listener:Function, useCapture:Boolean = false, priority:int = 0, useWeakReference:Boolean = false):void
        {
            mListeners.push({type:type, listener:listener, useCapture:useCapture});
            super.addEventListener(type, listener, useCapture, priority, useWeakReference);
        }

        /**
         * remove all registered event listeners
         */
        public function RemoveAllListeners():void
        {
            for each (var o:Object in mListeners)
            {
                this.removeEventListener(o.type, o.listener, o.useCapture);
            }
            mListeners = null;
        }
    }
}
