﻿package meyermike.core
{
    import flash.display.DisplayObjectContainer;
    import flash.utils.Dictionary;
    import flash.utils.getQualifiedClassName;

    /**
     * ...
     * @author Mike Meyer
     */
    public class GameObjectHolder
    {
        private var _container:DisplayObjectContainer;
        public function get Container():DisplayObjectContainer { return _container; }

        protected var _liveObjects:/*GameObject*/Array;
        public function get Objects():/*GameObject*/Array { return _liveObjects; }

        protected var mReusables:Dictionary;

        public function GameObjectHolder(container:DisplayObjectContainer)
        {
            _container = container;
            _liveObjects = new Array();
            mReusables = new Dictionary();
        }

        /**
         * add a game object to the holder.  unlike Add, it expects you to
         * provide a new instance and does not reuse objects
         */
        public function AddPremade(go:GameObject):void
        {
            Container.addChild(go);
            go.Init();
            Objects.push(go);
        }


        /// Add a game object to the holder.  Will try to re-use existing ones
        public function Add(type:Class):GameObject
        {
            var go:GameObject = FindReusableObject(type);
            if (null == go)
            {
                go = new type();
            }

            AddPremade(go);
            return go;
        }

        private function FindReusableObject(type:Class):GameObject
        {
            var classname:String = getQualifiedClassName(type);
            if (mReusables.hasOwnProperty(classname))
            {
                if (0 < mReusables[classname].length)
                {
                    return mReusables[classname].pop();
                }
            }

            return null;
        }

        public function Update(elapsedTime:Number):void
        {
            for each (var go:GameObject in Objects )
            {
                go.Update(elapsedTime);
            }

            RemoveDeadObjects();
        }

        public function Integrate(timeDelta:Number):void
        {
            for each (var go:GameObject in Objects )
            {
                go.Integrate(timeDelta);
            }
        }

        /**
         * call after all the final positions have been figured
         * just before drawing.
         * Calls PreDraw on each layer.
         */
        public function PreDraw():void
        {
            for each (var go:GameObject in Objects )
            {
                go.PreDraw();
            }
        }

        public function Beat(beat:Number):void
        {
            for each (var go:GameObject in Objects )
            {
                go.Beat(beat);
            }
        }

        public function OffBeat(beat:Number):void
        {
            for each (var go:GameObject in Objects )
            {
                go.OffBeat(beat);
            }
        }
        public function QuarterBeat(quarterBeat:Number):void
        {
            for each (var go:GameObject in Objects )
            {
                go.QuarterBeat(quarterBeat);
            }
        }

        private function RemoveDeadObjects():void
        {
            var removalQueue:Array = new Array();

            // look for any particles that need to be removed
            for (var i0:Number = 0; i0 < Objects.length; ++i0 )
            {
                var p0:GameObject = Objects[i0];

                // if it dies. queue it up for removal
                if (!p0.IsAlive())
                {
                    removalQueue.push(i0);
                }
            }

            // do any queued removals. looping backwards! important!
            for (var i1:Number = (removalQueue.length - 1); i1 >= 0 ; --i1)
            {
                RemoveAt(removalQueue[i1]);
            }
        }

        /**
         * remove the game object at index from the scene.  Store it in
         * reusables for later reuse.
         */
        private function RemoveAt(index:Number):void
        {
            var p:GameObject = Objects[index];
            p.Shutdown();

            var classname:String = getQualifiedClassName(p);

            // never reuse the base gameobject class, that will probably
            // always have custom stuff attached in it.
            if ("meyermike.core::GameObject" != classname)
            {
                if (!mReusables.hasOwnProperty(classname))
                {
                    mReusables[classname] = new Array();
                }
                mReusables[classname].push(p);
            }

            Container.removeChild(p);
            Objects.splice(index, 1);
        }

        /**
         * call a function for each go in the layer.  func should be of the form
         * function func(go:GameObject):void
         */
        public function ForEach(func:Function):void
        {
            for (var i:Number = 0; i < Objects.length; ++i )
            {
                func(Objects[i]);
            }
        }

        /**
         * Get rid of any and all objects held by this thing
         */
        public function RemoveAll():void
        {
            for (var i:Number = (Objects.length - 1); i >= 0 ; --i)
            {
                RemoveAt(i);
            }
        }
    }

}
