﻿package meyermike.util
{
    import meyermike.Global;

    /**
     * static functions to help with dealing with this game's faked perspective/3d
     * this is all based on
     * magic perspectve formula. © 1498 Leonardo da Vinvi
     *   apparent_size = ( 1 / ( 1 + ( (z - REALSIZE_Z) * _zscale) ) );
     *   _zscale = 2.0 * Math.tan(vertical_viewing_angle) / Global.StageWidth
     * but we actually store _zscale because it mostly makes things simpler
     * @author Mike Meyer
     */
    public class PerspectiveUtil
    {
        /// z value at which object's appear their "real" size
        private static const REALSIZE_Z:Number = 0;

        /// hard to say succinctly what it represents, but high values here make for a wider field of view
        private static var _zscale:Number = 0.0108253175; // about 60° vertical field of view when the screen is 320 pixels tall

        /// note that setting this changes most other properties
        /// also don't give it negative angles or angles approaching or
        /// greater than 0.5 * Math.PI
        public static function set VerticalFieldOfView(thetaInRadians:Number):void
        {
            _zscale = 2.0 * Math.tan(thetaInRadians) / meyermike.Global.StageHeight;
        }

        /// get the vertical field of view in radians
        public static function get VerticalFieldOfView():Number
        {
            return Math.atan(_zscale * 0.5 * meyermike.Global.StageHeight);
        }

        /// note that setting this changes most other properties
        /// also don't give it negative angles or angles approaching or
        /// greater than 0.5 * Math.PI
        public static function set HorizontalFieldOfView(thetaInRadians:Number):void
        {
            _zscale =  2.0 * Math.tan(thetaInRadians) / meyermike.Global.StageWidth;
        }

        /// get the horizontal field of view in radians
        public static function get HorizontalFieldOfView():Number
        {
            return Math.atan( _zscale * 0.5 * meyermike.Global.StageWidth);
        }

        /// find out the scaling value based on an object's depth.
        public static function GetScaleFromZ(z:Number):Number
        {
            return ( 1 / ( 1 + ( (z - REALSIZE_Z) * _zscale) ) );
        }

        /// inverse operation of the above, basically just for setting my z step based on the art
        public static function GetZFromScale(scale:Number):Number
        {
            return ((((1.0 / scale) - 1) / _zscale) + REALSIZE_Z);
        }
    }

}
