/**
 * Wrapper for Flash text fields to add convenient methods and
 * automatically set good defaults for this game.
 */
package meyermike.widget
{
    import flash.display.Sprite;
    import flash.text.TextField;
    import flash.text.TextFieldAutoSize;
    import flash.text.TextFieldType;
    import flash.text.TextFormat;
    import meyermike.constants.Defaults;
    import meyermike.core.GameObject;
    import meyermike.constants.Colors;

    public class TextField extends GameObject
    {
        /**
         * actual textfield to wrap
         */
        protected var mTextField:flash.text.TextField;

        public static const HALIGN_LEFT:uint       = 0;
        public static const HALIGN_CENTER:uint     = 1;
        public static const HALIGN_RIGHT:uint      = 2;
        public static const VALIGN_TOP:uint        = 3;
        public static const VALIGN_MIDDLE:uint     = 4;
        public static const VALIGN_BOTTOM:uint     = 5;

        private static const DEFAULT_WORDWRAP_WIDTH:Number = 200;

        private var _hAlign:uint;
        public function get hAlign():uint { return _hAlign; }
        public function set hAlign(value:uint):void
        {
            _hAlign = value;
            UpdateHAlign();
        }
        private var _vAlign:uint;
        public function get vAlign():uint { return _vAlign; }
        public function set vAlign(value:uint):void
        {
            _vAlign = value;
            UpdateVAlign();
        }

        public function get WordWrap():Boolean { return mTextField.wordWrap; }
        public function set WordWrap(value:Boolean ):void { mTextField.wordWrap = value; ResetSize(); }

        public function get WordWrapWidth():Number { return mTextField.width; }
        public function set WordWrapWidth(value:Number):void { mTextField.width = value; ResetSize(); }

        public function TextField():void
        {
            super();

            if (null == mTextField)
            {
                mTextField = new flash.text.TextField();
                mTextField.autoSize = TextFieldAutoSize.LEFT;
                mTextField.multiline = true;
                this.addChild(mTextField);
            }
            ResetDefaults();
        }

        /**
         * this function called for both initial setup and to re-init for reuse
         */
        public override function Init():void
        {
            super.Init();
            ResetDefaults();
        }

        /**
         * Change the displayed text.
         */
        public function SetText(desiredText:String):void
        {
            mTextField.text = desiredText;
            ResetSize();
        }

        /**
         * Change the displayed text.
         */
        public function SetHtmlText(desiredHtmlText:String):void
        {
            mTextField.htmlText = desiredHtmlText;
            ResetSize();
        }

        /**
         * Get the displayed text.
         */
        public function GetText():String
        {
            return mTextField.text;
        }

        /**
         * Change the font.
         */
        public function SetFont(font:String):void
        {
            var fmt:TextFormat = mTextField.getTextFormat();
            fmt.font = font;
            mTextField.defaultTextFormat = fmt;
            mTextField.setTextFormat(fmt);

            ResetSize();
        }

        /**
         * Change the displayed font size.
         */
        public function SetFontSize(desiredSize:Number):void
        {
            var fmt:TextFormat = mTextField.getTextFormat();
            fmt.size = desiredSize;
            mTextField.defaultTextFormat = fmt;
            mTextField.setTextFormat(fmt);

            ResetSize();
        }

        /**
         * used internally to keep .width and .height sensible
         */
        protected function ResetSize():void
        {
            UpdateHAlign();
            UpdateVAlign();
        }

        /**
         * center the textfield on some point.  CALL SETTEXT() *FIRST*
         */
        public function CenterOn(center_x:Number, center_y:Number):void
        {
            x = center_x;
            y = center_y;
            hAlign = HALIGN_CENTER;
            vAlign = VALIGN_MIDDLE;
        }

        /**
         * reset to defaults.  This thing will likely get re-used!
         */
        public function ResetDefaults():void
        {
            mTextField.x = 0;
            mTextField.y = 0;
            mTextField.selectable = false;
            mTextField.embedFonts = true;

            var format:TextFormat = new TextFormat();
            format.font = Defaults.FONT;
            format.size = Defaults.FONT_SIZE;
            mTextField.defaultTextFormat = format;
            mTextField.setTextFormat(format);
            mTextField.textColor = Defaults.FONT_COLOR;
            mTextField.text = "";

            hAlign = HALIGN_LEFT;
            vAlign = VALIGN_TOP;
            WordWrap = false;
            //WordWrapWidth = DEFAULT_WORDWRAP_WIDTH;
            ResetSize();
        }

        /**
         * Change the color of the displayed text
         */
        public function SetFontColor(color:Number):void
        {
            mTextField.textColor = color;
        }

        private function UpdateHAlign():void
        {
            switch (hAlign)
            {
                case HALIGN_LEFT:
                    mTextField.x = 0.0;
                    break;
                case HALIGN_CENTER:
                    mTextField.x = -0.5 * mTextField.textWidth;
                    break;
                case HALIGN_RIGHT:
                    mTextField.x = -1.0 * mTextField.textWidth;
                    break;
                default:
                    trace("unrecognized h alignment");
                    break;
            }
        }
        private function UpdateVAlign():void
        {
            switch (vAlign)
            {
                case VALIGN_TOP:
                    mTextField.y = 0;
                    break;
                case VALIGN_MIDDLE:
                    mTextField.y = -0.5 * mTextField.textHeight;
                    break;
                case VALIGN_BOTTOM:
                    mTextField.y = -mTextField.textHeight;
                    break;
                default:
                    trace("unrecognized v alignment");
                    break;
            }
        }
    }
}
